/*
 *  Ocean Shader by afl_ext
 *   MIT License
 *   https://www.shadertoy.com/view/MdXyzX
 *  Minor formatting changes and refactoring to improve readability
 *
 *  Key bindings:
 *  +/-     Eye/camera elevation
 *  w/W     Change wave depth
 *  d/D     Change wave drag
 *  r/R     Reset parameters
 *  arrows  Change view angle
 *  0       Reset view angle
 *  ESC     Exit
 */
#include "CSCIx239.h"
int move=1;      //  Moving light
int zh=0;        //  Light position
int th=0,ph=0;   //  View angles
float asp=1;     //  Aspect ratio
int shader=0;    //  Shader
float camy=1.5;  //  Camera elevation
float depth=1;   //  Wave Depth
float drag=0.28; //  How much waves pull on the water

//
//  Refresh display
//
void display(GLFWwindow* window)
{
   //  Erase the window and the depth buffer
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   //  Select shader
   glUseProgram(shader);
   //  Time
   float t = glfwGetTime();
   int id = glGetUniformLocation(shader,"time");
   glUniform1f(id,t);
   //  Screen size
   int width,height;
   glfwGetFramebufferSize(window,&width,&height);
   id = glGetUniformLocation(shader,"dim");
   glUniform2f(id,width,height);
   //  View direction (in radians)
   id = glGetUniformLocation(shader,"th");
   glUniform1f(id,th*M_PI/180);
   id = glGetUniformLocation(shader,"ph");
   glUniform1f(id,ph*M_PI/180);
   //  Camera elevation
   id = glGetUniformLocation(shader,"camy");
   glUniform1f(id,camy);
   //  Wave Depth
   id = glGetUniformLocation(shader,"depth");
   glUniform1f(id,depth);
   //  Wave Drag
   id = glGetUniformLocation(shader,"drag");
   glUniform1f(id,drag);
   //  Draw rectangle
   View(0,0,0,1);
   SetColor(1,1,1);
   glBegin(GL_QUADS);
   glVertex2f(-asp,-1);
   glVertex2f(+asp,-1);
   glVertex2f(+asp,+1);
   glVertex2f(-asp,+1);
   glEnd();
   //  Revert to fixed pipeline
   glUseProgram(0);

   //  Display parameters
   SetColor(1,1,1);
   glWindowPos2i(5,5);
   Print("Angle=%d,%d CamY=%.1f Wave Depth=%.1f Wave Drag=%.2f",th,ph,camy,depth,drag);
   //  Render the scene and make it visible
   ErrCheck("display");
   glFlush();
   glfwSwapBuffers(window);
}

//
//  Key pressed callback
//
void key(GLFWwindow* window,int key,int scancode,int action,int mods)
{
   //  Discard key releases (keeps PRESS and REPEAT)
   if (action==GLFW_RELEASE) return;

   //  Check for shift
   int shift = (mods & GLFW_MOD_SHIFT);

   //  Exit on ESC
   if (key == GLFW_KEY_ESCAPE)
      glfwSetWindowShouldClose(window,1);
   //  Reset view angle and location
   else if (key==GLFW_KEY_0)
      th = ph = 0;
   //  Increase/decrease asimuth
   else if (key==GLFW_KEY_RIGHT)
      th += 1;
   else if (key==GLFW_KEY_LEFT)
      th -= 1;
   //  Increase/decrease elevation
   else if (key==GLFW_KEY_UP)
      ph += 1;
   else if (key==GLFW_KEY_DOWN)
      ph -= 1;
   //  Camera elevation
   else if (key==GLFW_KEY_KP_ADD || key==GLFW_KEY_EQUAL)
      camy += 0.1;
   else if ((key==GLFW_KEY_KP_SUBTRACT || key==GLFW_KEY_MINUS) && camy>0.2)
      camy -= 0.1;
   //  Wave depth
   else if (key==GLFW_KEY_W && !shift)
      depth += 0.1;
   else if (key==GLFW_KEY_W && shift && depth>0.2)
      depth -= 0.1;
   //  Wave drag
   else if (key==GLFW_KEY_D && !shift)
      drag += 0.01;
   else if (key==GLFW_KEY_D && shift && drag>0.1)
      drag -= 0.01;
   else if (key==GLFW_KEY_R)
   {
      camy  = 1.5;
      depth = 1;
      drag  = 0.28;
   }

   //  Wrap angles
   th %= 360;
   ph %= 360;
   //  Update projection
   Projection(0,asp,1);
}

//
//  Window resized callback
//
void reshape(GLFWwindow* window,int width,int height)
{
   //  Get framebuffer dimensions (makes Apple work right)
   glfwGetFramebufferSize(window,&width,&height);
   //  Ratio of the width to the height of the window
   asp = (height>0) ? (double)width/height : 1;
   //  Set the viewport to the entire window
   glViewport(0,0, width,height);
   //  Set projection
   Projection(0,asp,1);
}

//
//  Main program with GLFW event loop
//
int main(int argc,char* argv[])
{
   //  Initialize GLFW
   GLFWwindow* window = InitWindow("Ocean Shader by afl_ext",1,600,600,&reshape,&key);

   //  Load shader
   shader = CreateShaderProg(NULL,"ocean.frag");

   //  Event loop
   ErrCheck("init");
   while(!glfwWindowShouldClose(window))
   {
      //  Light animation
      if (move) zh = fmod(90*glfwGetTime(),360);
      //  Display
      display(window);
      //  Process any events
      glfwPollEvents();
   }
   //  Shut down GLFW
   glfwDestroyWindow(window);
   glfwTerminate();
   return 0;
}
